import contextlib
import logging
import tkinter
from tkinter import font, messagebox, ttk, N, S, E, W

from chronophore import __title__, __version__, controller
from chronophore.config import CONFIG

logger = logging.getLogger(__name__)


class ChronophoreUI:
    """Simple Tkinter GUI for chronophore :
            - Entry for user id input
            - Button to sign in or out
            - List of currently signed in users
    """

    def __init__(self):
        self.root = tkinter.Tk()
        self.root.title('{} {}'.format(__title__, __version__))
        self.content = ttk.Frame(self.root, padding=(5, 5, 10, 10))

        # custom fonts
        self.large_font = font.Font(family='Helvetica', size=CONFIG['LARGE_FONT_SIZE'])
        self.medium_font = font.Font(family='Helvetica', size=CONFIG['MEDIUM_FONT_SIZE'])
        self.small_font = font.Font(family='Helvetica', size=CONFIG['SMALL_FONT_SIZE'])
        self.tiny_font = font.Font(family='Helvetica', size=CONFIG['TINY_FONT_SIZE'])

        self.large_header = self.large_font.copy()
        self.large_header.configure(weight='bold')

        self.tiny_header = self.tiny_font.copy()
        self.tiny_header.configure(weight='bold')

        # default widget fonts
        ttk.Style().configure('TLabel', font=self.medium_font)
        ttk.Style().configure('TButton', font=self.medium_font)
        # NOTE(amin): The following won't work for mysterious
        # reasons internal to tk:
        # ttk.Style().configure('TEntry', font=self.medium_font)

        # variables
        self.signed_in = tkinter.StringVar()
        self.user_id = tkinter.StringVar()
        self.feedback = tkinter.StringVar()

        # widgets
        self.frm_signedin = ttk.Frame(
            self.content,
            borderwidth=5,
            relief="sunken",
            width=200,
            height=100
        )
        self.lbl_signedin = ttk.Label(
            self.content,
            text="Currently Signed In:",
            font=self.tiny_header
        )
        self.lbl_signedin_list = ttk.Label(
            self.frm_signedin,
            textvariable=self.signed_in,
            font=self.tiny_font
        )
        self.lbl_welcome = ttk.Label(
            self.content,
            text=CONFIG['GUI_WELCOME_LABLE'],
            font=self.large_header
        )
        self.lbl_id = ttk.Label(
            self.content,
            text="Enter Student ID:",
        )
        self.ent_id = ttk.Entry(
            self.content,
            textvariable=self.user_id,
            font=self.small_font
        )
        self.lbl_feedback = ttk.Label(
            self.content,
            textvar=self.feedback,
        )
        self.btn_sign = ttk.Button(
            self.content,
            text="Sign In/Out",
            command=self._sign_in_button_press,
        )

        # assemble grid
        self.content.grid(column=0, row=0, sticky=(N, S, E, W))
        self.lbl_signedin.grid(column=0, row=0, pady=0)
        self.frm_signedin.grid(
            column=0, row=1, columnspan=1, rowspan=4, sticky=(N, S, E, W)
        )
        self.lbl_signedin_list.grid(column=0, row=0, columnspan=1, rowspan=4)
        self.lbl_welcome.grid(column=2, row=1, columnspan=1)
        self.lbl_id.grid(column=2, row=2, columnspan=1, sticky=(N))
        self.ent_id.grid(column=2, row=2, columnspan=1)
        self.lbl_feedback.grid(column=2, row=3)
        self.btn_sign.grid(column=2, row=4, columnspan=1, sticky=(N))

        # resize weights
        self.root.columnconfigure(0, weight=1, minsize=400)
        self.root.rowconfigure(0, weight=1, minsize=200)
        self.content.columnconfigure(0, weight=1, minsize=150)
        self.content.columnconfigure(1, weight=3)
        self.content.columnconfigure(2, weight=3)
        self.content.columnconfigure(3, weight=3)
        self.content.rowconfigure(0, weight=0)
        self.content.rowconfigure(1, weight=3, minsize=50)
        self.content.rowconfigure(2, weight=1, minsize=100)
        self.content.rowconfigure(3, weight=1)
        self.content.rowconfigure(4, weight=3)

        # key bindings
        self.root.bind('<Return>', self._sign_in_button_press)
        self.root.bind('<KP_Enter>', self._sign_in_button_press)

        self.ent_id.focus()
        self._set_signed_in()
        self.root.mainloop()

    def _set_signed_in(self):
        """Populate the signed_in list with the names of
        currently signed in users.
        """
        names = [
            controller.get_user_name(user, full_name=CONFIG['FULL_USER_NAMES'])
            for user in controller.signed_in_users()
        ]
        self.signed_in.set('\n'.join(sorted(names)))

    def _show_feedback_label(self, message, seconds=None):
        """Display a message in lbl_feedback, which then times out
        after some number of seconds. Use after() to schedule a callback
        to hide the feedback message. This works better than using threads,
        which can cause problems in Tk.
        """
        if seconds is None:
            seconds = CONFIG['MESSAGE_DURATION']

        # cancel any existing callback to clear the feedback
        # label. this prevents flickering and inconsistent
        # timing during rapid input.
        with contextlib.suppress(AttributeError):
            self.root.after_cancel(self.clear_feedback)

        logger.debug('Label feedback: "{}"'.format(message))
        self.feedback.set(message)
        self.clear_feedback = self.root.after(
            1000 * seconds, lambda: self.feedback.set("")
        )

    def _show_confirm_window(self, message):
        logger.debug('Window feedback: "{}"'.format(message))
        yes_pressed = messagebox.askyesno(
            message=message,
            title='{} notification'.format(__title__),
            icon='question',
            default='yes',
            parent=self.root,
        )
        logger.debug('Sign in confirmed: {}'.format(yes_pressed))
        return yes_pressed

    def _sign_in_button_press(self, *args):
        """Validate input from ent_id, then sign in to the Timesheet."""
        user_id = self.ent_id.get().strip()

        try:
            status = controller.sign(user_id)

        # ERROR: User type is unknown
        except ValueError as e:
            logger.error(e, exc_info=True)
            messagebox.showerror(message=e)

        # ERROR: User is unregistered
        except controller.UnregisteredUser as e:
            logger.debug(e)
            messagebox.showerror(message=e)

        # User needs to select type
        except controller.AmbiguousUserType as e:
            logger.debug(e)
            user_type = UserTypeSelectionDialog(self).show()
            if user_type:
                status = controller.sign(user_id, user_type=user_type)
                self._show_feedback_label(
                    'Signed {}: {}'.format(status.in_or_out, status.user_name)
                )

        # User has signed in or out normally
        else:
            sign_choice_is_confirmed = self._show_confirm_window(
                'Sign {}: {}?'.format(status.in_or_out, status.user_name)
            )

            if not sign_choice_is_confirmed:
                # Undo sign-in or sign-out
                if status.in_or_out == 'in':
                    controller.undo_sign_in(status.entry)
                elif status.in_or_out == 'out':
                    controller.undo_sign_out(status.entry)
            else:
                self._show_feedback_label(
                    'Signed {}: {}'.format(status.in_or_out, status.user_name)
                )

        finally:
            self._set_signed_in()
            self.ent_id.delete(0, 'end')
            self.ent_id.focus()


class UserTypeSelectionDialog:
    """A dialog box that prompts the user to select
    which user type to sign in with. Their choice is
    returned through the show() method.
    """

    def __init__(self, parent):
        self.toplevel = tkinter.Toplevel(parent.root)
        self.toplevel.transient(parent.root)
        self.toplevel.title('User Type Selection')
        self.parent = parent
        self.frame = ttk.Frame(self.toplevel, padding=(5, 5, 10, 10))

        # variables
        self.user_type = tkinter.StringVar()

        # widgets
        self.lbl_message = ttk.Label(
            self.frame,
            text='Sign in as:',
        )
        self.rb_student = ttk.Radiobutton(
            self.frame,
            text='Student',
            variable=self.user_type,
            value='student',
        )
        self.rb_tutor = ttk.Radiobutton(
            self.frame,
            text='Tutor',
            variable=self.user_type,
            value='tutor',
        )
        self.btn_ok = ttk.Button(
            self.frame,
            text='Ok',
            command=self._ok,
        )
        self.btn_cancel = ttk.Button(
            self.frame,
            text='Cancel',
            command=self._cancel,
        )

        # assemble grid
        self.frame.grid(column=0, row=0, sticky=(N, S, E, W))
        self.lbl_message.grid(column=0, row=0, columnspan=2, sticky=(W, E))
        self.rb_student.grid(column=0, row=1, columnspan=2, sticky=W)
        self.rb_tutor.grid(column=0, row=2, columnspan=2, sticky=W)
        self.btn_ok.grid(column=0, row=3)
        self.btn_cancel.grid(column=1, row=3)

        # 'Tutor' is selected by default
        self.rb_tutor.invoke()

        self.toplevel.protocol('WM_DELETE_WINDOW', self._cancel)

        # key bindings
        self.toplevel.bind('<Return>', self._ok)
        self.toplevel.bind('<KP_Enter>', self._ok)
        self.toplevel.bind('<Escape>', self._cancel)

    def _ok(self, *args):
        self.toplevel.destroy()

    def _cancel(self, *args):
        self.user_type.set(None)
        self.toplevel.destroy()

    def show(self):
        """Claim application-wide focus. Return user type
        based the currently selected radio button.
        """
        self.toplevel.grab_set()
        self.toplevel.wait_window(self.toplevel)

        user_type = self.user_type.get()
        logger.debug('User type selected: {}'.format(user_type))

        if user_type == 'student' or user_type == 'tutor':
            return user_type
        else:
            return None



"""Chronophore is a simple time-tracking program. It keeps track of
users' hours as they sign in and out. Data is stored in a
human-readable json file.

This project was started to help keep track of students signing in
and out at a tutoring program in a community college, but should be
adaptable to other use cases.
"""
from sqlalchemy.orm import sessionmaker

__title__ = 'chronophore'
__version__ = '0.5.1'
__license__ = 'MIT'
__author__ = 'Amin Mesbah'
__email__ = 'mesbahamin@gmail.com'
__description__ = 'Desktop app for tracking sign-ins and sign-outs in a tutoring center.'

Session = sessionmaker()



import appdirs
import argparse
import logging
import os
import pathlib
from sqlalchemy import create_engine

from chronophore import (
    __description__, __title__, __version__, controller, Session
)
from chronophore.models import Base, add_test_users
from chronophore.view import ChronophoreUI


def get_args():
    parser = argparse.ArgumentParser(
        prog=__title__,
        description=__description__,
    )
    parser.add_argument(
        '--testdb', action='store_true',
        help='create and use a database with test users'
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='print a detailed log'
    )
    parser.add_argument(
        '--debug', action='store_true',
        help='print debug log'
    )
    parser.add_argument(
        '-V', '--version', action='store_true',
        help='print version info and exit'
    )
    return parser.parse_args()


def set_up_logging(log_file, console_log_level):
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    fh = logging.FileHandler(str(log_file))
    fh.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(console_log_level)
    formatter = logging.Formatter(
        "{asctime} {levelname} ({name}): {message}", style='{'
    )
    fh.setFormatter(formatter)
    ch.setFormatter(formatter)
    logger.addHandler(fh)
    logger.addHandler(ch)

    return logger


def main():
    args = get_args()

    # Make Chronophore's directories and files in $HOME
    DATA_DIR = pathlib.Path(appdirs.user_data_dir(__title__))
    LOG_FILE = pathlib.Path(appdirs.user_log_dir(__title__), 'debug.log')
    os.makedirs(str(DATA_DIR), exist_ok=True)
    os.makedirs(str(LOG_FILE.parent), exist_ok=True)

    if args.version:
        print('{} {}'.format(__title__, __version__))
        raise SystemExit

    if args.debug:
        CONSOLE_LOG_LEVEL = logging.DEBUG
    elif args.verbose:
        CONSOLE_LOG_LEVEL = logging.INFO
    else:
        CONSOLE_LOG_LEVEL = logging.WARNING

    logger = set_up_logging(LOG_FILE, CONSOLE_LOG_LEVEL)
    logger.debug('-'*80)
    logger.info('{} {}'.format(__title__, __version__))
    logger.debug('Log File: {}'.format(LOG_FILE))
    logger.debug('Data Directory: {}'.format(DATA_DIR))

    if args.testdb:
        DATABASE_FILE = DATA_DIR.joinpath('test.sqlite')
        logger.info('Using test database.')
    else:
        DATABASE_FILE = DATA_DIR.joinpath('chronophore.sqlite')

    logger.debug('Database File: {}'.format(DATABASE_FILE))
    engine = create_engine('sqlite:///{}'.format(str(DATABASE_FILE)))
    Base.metadata.create_all(engine)
    Session.configure(bind=engine)

    if args.debug:
        logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)

    if args.testdb:
        add_test_users(session=Session())

    controller.flag_forgotten_entries(session=Session())

    ChronophoreUI()

    logger.debug('{} stopping'.format(__title__))



import logging
from datetime import date
from sqlalchemy import event, Boolean, Column, Date, ForeignKey, String
from sqlalchemy.dialects.sqlite import TIME
from sqlalchemy.engine import Engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

logger = logging.getLogger(__name__)

Base = declarative_base()
SQLite_Time = TIME(storage_format='%(hour)02d:%(minute)02d:%(second)02d')


@event.listens_for(Engine, "connect")
def set_sqlite_pragma(dbapi_connection, connection_record):
    """Upon every db connection, issue a command
    to ensure foreign key constraints are enforced.

    This is a sqlite-specific issue:
    http://stackoverflow.com/questions/2614984/sqlite-sqlalchemy-how-to-enforce-foreign-keys
    http://docs.sqlalchemy.org/en/latest/dialects/sqlite.html#foreign-key-support
    """
    cursor = dbapi_connection.cursor()
    cursor.execute("PRAGMA foreign_keys=ON")
    cursor.close()


class User(Base):
    """Schema for the 'users' table."""
    __tablename__ = 'users'

    user_id = Column(String, primary_key=True, unique=True)

    date_joined = Column(Date)
    date_left = Column(Date, nullable=True)
    education_plan = Column(Boolean, default=False)
    school_email = Column(String, nullable=True)
    personal_email = Column(String, nullable=True)
    first_name = Column(String)
    last_name = Column(String)
    major = Column(String, nullable=True)
    is_student = Column(Boolean, default=True)
    is_tutor = Column(Boolean, default=False)

    entries = relationship('Entry', back_populates='user', lazy='dynamic')

    def __repr__(self):
        return (
            'User('
            + 'user_id={},'.format(self.user_id)
            + ' date_joined={},'.format(self.date_joined)
            + ' date_left={},'.format(self.date_left)
            + ' education_plan={},'.format(self.education_plan)
            + ' school_email={},'.format(self.school_email)
            + ' personal_email={},'.format(self.personal_email)
            + ' first_name={},'.format(self.first_name)
            + ' last_name={},'.format(self.last_name)
            + ' major={},'.format(self.major)
            + ' is_student={},'.format(self.is_student)
            + ' is_tutor={},'.format(self.is_tutor)
            + ')'
        )


class Entry(Base):
    """Schema for the 'timesheet' table."""
    __tablename__ = 'timesheet'

    uuid = Column(String, primary_key=True, unique=True)

    date = Column(Date)
    forgot_sign_out = Column(Boolean, default=False)
    time_in = Column(SQLite_Time)
    time_out = Column(SQLite_Time)
    user_id = Column(String, ForeignKey('users.user_id'), nullable=False)
    user_type = Column(String, nullable=False)

    user = relationship('User', back_populates='entries')

    def __repr__(self):
        return (
            'Entry('
            + 'uuid={},'.format(self.uuid)
            + ' date={},'.format(self.date)
            + ' forgot_sign_out={},'.format(self.forgot_sign_out)
            + ' time_in={},'.format(self.time_in)
            + ' time_out={},'.format(self.time_out)
            + ' user_id={},'.format(self.user_id)
            + ' user_type={},'.format(self.user_type)
            + ')'
        )


def add_test_users(session):
    """Add two hobbits and a wizard to the
    users table for testing purposes. These
    are not necessarily the same test users
    as in the unit tests.

    This function is idempotent.
    """
    test_users = [
        User(
            user_id='000000000',
            date_joined=date(2014, 12, 11),
            date_left=None,
            education_plan=False,
            personal_email='baggins.frodo@gmail.com',
            first_name='Frodo',
            last_name='Baggins',
            major='Medicine',
            is_student=True,
            is_tutor=True,
        ),
        User(
            user_id='000111111',
            date_joined=date(2015, 2, 16),
            date_left=None,
            education_plan=True,
            personal_email='gamgee.samwise@gmail.com',
            first_name='Sam',
            last_name='Gamgee',
            major='Agriculture',
            is_student=True,
            is_tutor=False,
        ),
        User(
            user_id='000222222',
            date_joined=date(2010, 10, 10),
            date_left=None,
            education_plan=False,
            personal_email='mithrandir@gmail.com',
            first_name='Gandalf',
            last_name='the Grey',
            major='Computer Science',
            is_student=False,
            is_tutor=True,
        ),
    ]

    registered_ids = {
        user_id for (user_id, )
        in session.query(User.user_id).all()
    }

    for user in test_users:
        if user.user_id not in registered_ids:
            session.add(user)
            name = ' '.join([user.first_name, user.last_name])
            logger.info(
                'Adding test user: {} ({}).'.format(user.user_id, name)
            )

    session.commit()



import collections
import logging
import uuid
from datetime import date, datetime

from chronophore import Session
from chronophore.models import Entry, User

logger = logging.getLogger(__name__)


class AmbiguousUserType(Exception):
    """This exception is raised when a user
    with multiple user types tries to sign in.
    """
    def __init__(self, message):
        super().__init__(message)
        self.message = message


class UnregisteredUser(Exception):
    """This exception is raised when a user
    id doesn't match any user in the database.
    """
    def __init__(self, message):
        super().__init__(message)
        self.message = message


# Status is used by the sign() function to
# return relevant information to the gui.
Status = collections.namedtuple(
    'Status',
    [
        'valid',
        'in_or_out',
        'user_name',
        'user_type',
        'entry',
    ]
)


def flag_forgotten_entries(session, today=None):
    """Flag any entries from previous days where
    users forgot to sign out.
    """
    today = date.today() if today is None else today

    forgotten = session.query(Entry).filter(
            Entry.time_out.is_(None)).filter(
            Entry.forgot_sign_out.is_(False)).filter(
            Entry.date < today)

    for entry in forgotten:
        e = sign_out(entry, forgot=True)
        logger.debug('Signing out forgotten entry: {}'.format(e))
        session.add(e)

    session.commit()


def signed_in_users(session=None, today=None, full_name=True):
    """Return list of names of currently signed in users.
    Full names by default.
    """
    if session is None:
        session = Session()
    else:
        session = session

    if today is None:
        today = date.today()
    else:
        today = today

    signed_in_users = session.query(User).filter(
            Entry.date == today).filter(
            Entry.time_out.is_(None)).filter(
            User.user_id == Entry.user_id).all()

    session.close()
    return signed_in_users


def get_user_name(user, full_name=True):
    """Return the user's name as a string.
    Full names by default.
    """
    try:
        if full_name:
            name = ' '.join([user.first_name, user.last_name])
        else:
            name = user.first_name
    except AttributeError:
        name = None

    return name


def sign_in(user, user_type=None, date=None, time_in=None):
    """Add a new entry to the timesheet."""
    now = datetime.today()
    if date is None:
        date = now.date()
    if time_in is None:
        time_in = now.time()
    if user_type is None:
        if user.is_student and user.is_tutor:
            raise AmbiguousUserType('User is both a student and a tutor.')
        elif user.is_student:
            user_type = 'student'
        elif user.is_tutor:
            user_type = 'tutor'
        else:
            raise ValueError('Unknown user type.')

    new_entry = Entry(
        uuid=str(uuid.uuid4()),
        date=date,
        time_in=time_in,
        time_out=None,
        user_id=user.user_id,
        user_type=user_type,
        user=user,
    )

    logger.info('{} ({}) signed in.'.format(new_entry.user_id, new_entry.user_type))
    return new_entry


def sign_out(entry, time_out=None, forgot=False):
    """Sign out of an existing entry in the timesheet.
    If the user forgot to sign out, flag the entry.
    """
    if time_out is None:
        time_out = datetime.today().time()

    if forgot:
        entry.forgot_sign_out = True
        logger.info(
            '{} forgot to sign out on {}.'.format(entry.user_id, entry.date)
        )

    else:
        entry.time_out = time_out

    logger.info('{} ({}) signed out.'.format(entry.user_id, entry.user_type))
    return entry


def undo_sign_in(entry, session=None):
    """Delete a signed in entry."""
    if session is None:
        session = Session()
    else:
        session = session

    entry_to_delete = session.query(Entry).filter(
            Entry.uuid == entry.uuid).one_or_none()

    if entry_to_delete:
        logger.info('Undo sign in: {}'.format(entry_to_delete.user_id))
        logger.debug('Undo sign in: {}'.format(entry_to_delete))
        session.delete(entry_to_delete)
        session.commit()
    else:
        error_message = 'Entry not found: {}'.format(entry)
        logger.error(error_message)
        raise ValueError(error_message)


def undo_sign_out(entry, session=None):
    """Sign in a signed out entry."""
    if session is None:
        session = Session()
    else:
        session = session

    entry_to_sign_in = session.query(Entry).filter(
            Entry.uuid == entry.uuid).one_or_none()

    if entry_to_sign_in:
        logger.info('Undo sign out: {}'.format(entry_to_sign_in.user_id))
        logger.debug('Undo sign out: {}'.format(entry_to_sign_in))
        entry_to_sign_in.time_out = None
        session.add(entry_to_sign_in)
        session.commit()
    else:
        error_message = 'Entry not found: {}'.format(entry)
        logger.error(error_message)
        raise ValueError(error_message)


def sign(user_id, user_type=None, today=None, session=None):
    """Check user id for validity, then sign user in or out
    depending on whether or not they are currently signed in.

    Return:
        - status: A string reporting the result of the sign
        attempt.
    """
    if session is None:
        session = Session()
    else:
        session = session

    if today is None:
        today = date.today()
    else:
        today = today

    user = session.query(User).filter(User.user_id == user_id).one_or_none()

    if user:
        signed_in_entries = user.entries.filter(
                Entry.date == today).filter(
                Entry.time_out.is_(None)).all()

        if not signed_in_entries:
            new_entry = sign_in(user, user_type=user_type)
            session.add(new_entry)
            status = Status(
                valid=True,
                in_or_out='in',
                user_name=get_user_name(user),
                user_type=new_entry.user_type,
                entry=new_entry
            )

        else:
            for entry in signed_in_entries:
                signed_out_entry = sign_out(entry)
                session.add(signed_out_entry)
                status = Status(
                    valid=True,
                    in_or_out='out',
                    user_name=get_user_name(user),
                    user_type=signed_out_entry.user_type,
                    entry=signed_out_entry
                )

        session.commit()

    else:
        raise UnregisteredUser(
            '{} not registered. Please register at the front desk.'.format(
                user_id
            )
        )

    logger.debug(status)
    return status



#!/usr/bin/python3

import argparse
import json
import logging
import pathlib
import sqlalchemy

from datetime import datetime
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from chronophore.models import Base, Entry, User


def get_args():
    parser = argparse.ArgumentParser(
        description='Load Chronophore json files into a sqlite database.'
    )
    parser.add_argument(
        'files',
        help='json file(s) to add to the database',
        type=str,
        nargs='+'
    )
    parser.add_argument(
        '-t', '--type',
        required=True,
        choices=['users', 'timesheet'],
        help='type of data stored in json file(s)'
    )
    parser.add_argument(
        '-o', '--output',
        help='path of sqlite database (default: output.sqlite)'
    )
    parser.add_argument(
        '--date-format',
        default='%Y-%m-%d',
        help='format string for dates in json data'
    )
    parser.add_argument(
        '--time-format',
        default='%H:%M:%S',
        help='format string for dates in json data'
    )
    parser.add_argument(
        '-n', '--dry-run', action='store_true',
        help='perform a trial run with no changes made'
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='print a detailed log'
    )

    return parser.parse_args()


def make_entry(json_item, time_format, date_format):
    uuid, entry_info = json_item

    date = datetime.strptime(entry_info['date'], date_format).date()
    time_in = datetime.strptime(entry_info['time_in'], time_format).time()

    if entry_info['time_out'] is not None:
        time_out = datetime.strptime(entry_info['time_out'], time_format).time()
    else:
        time_out = None

    user_id = entry_info['user_id']

    return Entry(
        uuid=uuid,
        date=date,
        time_in=time_in,
        time_out=time_out,
        user_id=user_id,
        user_type='student',
    )


def make_user(json_item, date_format):
    user_id, user_info = json_item

    if user_info['Date Joined'] is not None:
        date_joined = datetime.strptime(
                user_info['Date Joined'],
                date_format).date()
    else:
        date_joined = None

    if user_info['Date Left'] is not None:
        date_left = datetime.strptime(
                user_info['Date Joined'],
                date_format).date()
    else:
        date_left = None

    education_plan = user_info['Education Plan']
    school_email = user_info['School Email']
    personal_email = user_info['Personal Email']
    first_name = user_info['First Name']
    last_name = user_info['Last Name']
    major = user_info['Major']

    return User(
        user_id=user_id,
        date_joined=date_joined,
        date_left=date_left,
        education_plan=education_plan,
        school_email=school_email,
        personal_email=personal_email,
        first_name=first_name,
        last_name=last_name,
        major=major,
    )


def main():
    args = get_args()

    DRY_RUN = args.dry_run

    if args.verbose:
        LOGGING_LEVEL = logging.DEBUG
    else:
        LOGGING_LEVEL = logging.INFO

    logging.basicConfig(
        level=LOGGING_LEVEL,
        format='%(levelname)s:%(asctime)s: %(message)s'
    )

    if DRY_RUN:
        logging.info('Starting test run...')

    JSON_FILES = [pathlib.Path(f) for f in args.files]

    if args.output:
        DATABASE_FILE = args.output
    else:
        DATABASE_FILE = 'output.sqlite'

    engine = create_engine('sqlite:///{}'.format(DATABASE_FILE))
    Session = sessionmaker(bind=engine)
    Base.metadata.create_all(engine)

    TYPE = args.type
    DATE_FORMAT = args.date_format
    TIME_FORMAT = args.time_format

    session = Session()
    for json_file in JSON_FILES:
        try:
            with json_file.open('r') as f:
                data = json.load(f)

                for json_item in data.items():
                    if TYPE == 'timesheet':
                        db_item = make_entry(json_item, TIME_FORMAT, DATE_FORMAT)
                        logging.debug('Adding timesheet entry:\n\t{}'.format(
                            db_item))
                    elif TYPE == 'users':
                        db_item = make_user(json_item, DATE_FORMAT)
                        logging.debug('Adding user:\n\t{}'.format(db_item))

                    session.add(db_item)
                    logging.debug('session.add(db_item)')

        except FileNotFoundError as e:
            logging.error('File not found: {}'.format(json_file))
            logging.debug(e)
            logging.info('Cancelling. No data commited to database.')
            raise SystemExit

        except KeyError as e:
            logging.error("Invalid json data for type '{}' in {}".format(
                TYPE, json_file))
            logging.debug(e)
            logging.info('Cancelling. No data commited to database.')
            raise SystemExit

    try:
        if not DRY_RUN:
            session.commit()

    except sqlalchemy.exc.IntegrityError as e:
        bad_items = '\n\t'.join(str(p) for p in e.params)
        logging.error('{}:\n\t{}'.format(e.orig, bad_items))
        logging.debug(e)
        logging.info('No data commited to database.')

    else:
        if not DRY_RUN:
            logging.info('Data successfully commited to database.')
        else:
            logging.info('Finishing test run.\nNo data commited to database.')

    finally:
        session.close()


if __name__ == '__main__':
    main()



import appdirs
import configparser
import logging
import os
import pathlib
from collections import OrderedDict

from chronophore import __title__

logger = logging.getLogger(__name__)


def _load_config(config_file):
    """Load settings from config file and return them as
    a dict.
    """
    logger.debug('Config file: {}'.format(config_file))

    parser = configparser.ConfigParser()
    try:
        with config_file.open('r') as f:
            parser.read_file(f)

    except FileNotFoundError as e:
        logger.warning('Config file not found')
        parser = _use_default(config_file)

    except configparser.ParsingError as e:
        logger.warning('Error in config file: {}'.format(e))
        parser = _use_default(config_file)

    finally:
        try:
            config = _load_options(parser)
        except (configparser.NoOptionError):
            parser = _use_default(config_file)
            config = _load_options(parser)

        logger.debug('Config loaded: {}'.format(config_file))
        return config


def _load_options(parser):
    config = dict(
        MESSAGE_DURATION=parser.getint('gui', 'message_duration'),
        GUI_WELCOME_LABLE=parser.get('gui', 'gui_welcome_label'),
        FULL_USER_NAMES=parser.getboolean('gui', 'full_user_names'),
        LARGE_FONT_SIZE=parser.getint('gui', 'large_font_size'),
        MEDIUM_FONT_SIZE=parser.getint('gui', 'medium_font_size'),
        SMALL_FONT_SIZE=parser.getint('gui', 'small_font_size'),
        TINY_FONT_SIZE=parser.getint('gui', 'tiny_font_size'),
    )
    return config


def _use_default(config_file):
    """Write default values to a config file. Return
    a ConfigParser object with the values loaded.
    """

    default_config = OrderedDict((
        (
            'gui',
            OrderedDict(
                (
                    ('message_duration', 5),
                    ('gui_welcome_label', 'Welcome to the STEM Learning Center!'),
                    ('full_user_names', True),
                    ('large_font_size', 30),
                    ('medium_font_size', 18),
                    ('small_font_size', 15),
                    ('tiny_font_size', 10),
                )
            ),
        ),
    ))

    parser = configparser.ConfigParser()
    parser.read_dict(default_config)

    if config_file.exists():
        backup = config_file.with_suffix('.bak')
        os.rename(str(config_file), str(backup))
        logger.info('{} moved to {}.'.format(config_file, backup))

    with config_file.open('w') as f:
        parser.write(f)

    logger.info('Default config file created.')

    return parser


CONFIG_FILE = pathlib.Path(appdirs.user_config_dir(__title__), 'config.ini')
os.makedirs(str(CONFIG_FILE.parent), exist_ok=True)
CONFIG = _load_config(CONFIG_FILE)



#!/usr/bin/python3

import argparse
import logging
import pathlib
import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from chronophore.models import Base, Entry

__description__ = """
Update Chronophore database to be compatible with a new version.
"""


def migrate_050_to_051(session):
    """Set time_out field of all flagged
    timesheet entries to Null.
    """
    entries_to_update = session.query(Entry).filter(
            Entry.forgot_sign_out.is_(True)).filter(
            Entry.time_out.isnot(None))

    for entry in entries_to_update:
        entry.time_out = None
        logging.info('Entry updated {}'.format(entry.uuid))
        logging.debug(entry.uuid)
        session.add(entry)


def get_args():
    parser = argparse.ArgumentParser(
        description=__description__
    )
    parser.add_argument(
        'database',
        help='Chronophore database to update in-place',
    )
    parser.add_argument(
        '-n', '--dry-run', action='store_true',
        help='perform a trial run with no changes made'
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='print a detailed log'
    )

    return parser.parse_args()


def main():
    args = get_args()

    DRY_RUN = args.dry_run

    if args.verbose:
        LOGGING_LEVEL = logging.DEBUG
    else:
        LOGGING_LEVEL = logging.INFO

    logging.basicConfig(
        level=LOGGING_LEVEL,
        format='%(levelname)s:%(asctime)s: %(message)s'
    )

    if DRY_RUN:
        logging.info('Starting test run...')

    DATABASE_FILE = pathlib.Path(args.database)

    engine = create_engine('sqlite:///{}'.format(DATABASE_FILE))
    Session = sessionmaker(bind=engine)
    Base.metadata.create_all(engine)

    session = Session()

    migrate_050_to_051(session)

    try:
        if not DRY_RUN:
            session.commit()

    except sqlalchemy.exc.IntegrityError as e:
        bad_items = '\n\t'.join(str(p) for p in e.params)
        logging.error('{}:\n\t{}'.format(e.orig, bad_items))
        logging.debug(e)
        logging.info('No data commited to database.')

    else:
        if not DRY_RUN:
            logging.info('Data successfully commited to database.')
        else:
            logging.info('Finishing test run.\nNo data commited to database.')

    finally:
        session.close()


if __name__ == '__main__':
    main()
